#!/bin/bash 					
#SBATCH -J w99_BENZENE_DMACRYS
#SBATCH -N 1
#SBATCH -n 12
#SBATCH -p exclusive 
#SBATCH -t 24:00:00

sleep 10
for writelogfile in 1;
do
######################################################################################################################################################################################################################################################################################
#                                                                                                                                                                                                                                                                                    #
# Simple script to set up and run DMACRYS calculations                                                                                                                                                                                                                               #
#                                                                                                                                                                                                                                                                                    #
# The top 6 lines need changing or removing before running they are platform dependent.                                                                                                                                                                                              #
#                                                                                                                                                                                                                                                                                    #
# This Script can be submitted as a batch job to a cluster and automates a large portion of the DMACRYS calculation. It is recommended that a user unfamiliar with DMACRYS familiarised themselves with the process before using this script. Below are a list of things which must  #
# be in the current working directory. The only non standard file which required is the {mol}.dsp (eg. benzene.dsp). This file specifies the atoms which are required to determine an internal axis, three atoms are required. Three ways to get the atom numbers required are below #
#                                                                                                                                                                                                                                                                                    #
# Current working directory must contain:                                                                                                                                                                                                                                            #
#	 - ${mol}.fdat     ... input file                                                                                                                                                                                                                                                #
#	 - cutoff          ... dmacrys 'cutoff' file                                                                                                                                                                                                                                     #
#	 - ${mol}.dsp      ... file giving the numbers of three atoms defining line and plane, e.g. "C1 C2 C3"                                                                                                                                                                           #
#    - fitcrys.pots    ... potentials file for the Buckingham potential                                                                                                                                                                                                              #
#                                                                                                                                                                                                                                                                                    #
# Notes  - ${mol}.dsp must be created by the user.                                                                                                                                                                                                                                   #
#		>> cp ${mol}.fdat ${mol}.dat                                                                                                                                                                                                                                                 #
#		>> rpluto ${mol}                                                                                                                                                                                                                                                             #
#		>> (click "labl")                                                                                                                                                                                                                                                            #
#		>> write to ${mol}.dsp the atom labels of three atoms that define a line and a plane.                                                                                                                                                                                        #
#       >> Alternatively Run neighcrys once initially with just the fdat or res file with a cutoff file. In fort.21 find the geometry and atom numbers. Write these to a file rite a new file ${mol}.dsp                                                                             #
#       >> Or  map the atom numbers from *.nem to *.nnl file and get the atom number from there. Write a new file ${mol}.dsp containing the element label and the atom number. Example C1 C2 C3.                                                                                     #                                                                                               #                                                                                                                                                                                     #
#                                                                                                                                                                                                                                                                                    #
#          Check the file path on line number 223 changes are probably needed. Delete or comment out lines 161 to 164 and change all fort_template#W99.22 files to fort_template#.22 to use with the FIT potential. Currently set to run with the W99 potential. Currently set to    #  
#          run Gaussian 03 search g03run to change.                                                                                                                                                                                                                                  #
#                                                                                                                                                                                                                                                                                    #
#                                                                                                                                                                                                                                                                                    #
# Usage:                                                                                                                                                                                                                                                                             #
#	 - qsub dmacrys_setup.sub | tee dmacrys_setup.jobid                                                                                                                                                                                                                              #
#                                                                                                                                                                                                                                                                                    #
# Originally developed by D.S.Palmer, Max Planck Institute for Mathematics in the Sciences/University of Strathclyde. Edited by J.L. McDonagh, University of St Andrews, for use with the W99 potential.                                                                             #
#                                                                                                                                                                                                                                                                                    #
######################################################################################################################################################################################################################################################################################

#User-Defined Variables
dmacrysdir=/DMACRYS_Scripts  					#Directory containing scripts and templates/ directory
formchk=/g03/formchk							#formchk executable
neighcrys=neighcrys.out 						#neighcrys executable
dmacrys=dmacrys.out								#dmacrys executable
gdma=gdma										#gdma executable
fvibsetup=FvibSetup.pl							#Graeme Day's perl script for setting up lattice entropy calculations
elastphon=elastphon.out							#Graeme Day's perl script for calculating lattice entropies
ext=dat                        					#either "fdat" or "res".  
g03method=MP2/6-31G**			   				#Level of QM theory for DMA.  Must match names of variables in gaussian_setup_dmacrys.py
cutoff=cutoff									#dmacrys cutoff file
mol=BENZENE 	 								#name or ID of molecule, e.g. $mol.fdat or $mol.res = name.res .



#Other variables (do not edit)
home=`pwd`

##################################
#
# Step 1: Find the atom labels
#
##################################

echo "Step 1: Finding atom labels ..."

if [ ! -d neighcrys1 ]; then mkdir neighcrys1; fi
cp cutoff $mol.$ext ${mol}.dsp neighcrys1
cd neighcrys1 
sed "s/DSPMOL/$mol/g" $dmacrysdir/fort_template1W99.22 > fort.22
sed -i "s/DSPEXT/$ext/g" fort.22

#Run neighcrys using non-interactive mode (commands are read from fort.22)
$neighcrys << EOF
n
EOF

#Un-useless neighcrys (which prints either ${mol}.res.dmain or ${mol}.dmain, depending on whether the input file is in .res or .fdat format)
for i in `ls *.$ext.*`; do j=`echo $i | sed 's/\.'$ext'//g'`; mv $i $j; done

#Run dmacrys
$dmacrys < $mol.dmain > $mol.dmaout

echo "Step 1: FINISHED!"

##################################
#
# Step 2: Set up axes file
#
##################################

echo "Step 2: Setting up axes file ..."

a1=`head -1 ${mol}.dsp | awk '{print $1}'`
a2=`head -1 ${mol}.dsp | awk '{print $2}'`
a3=`head -1 ${mol}.dsp | awk '{print $3}'`

#HACK - the "-a" option causes grep to treat binary files as text. 'Tis a hack.  For some reason, bash sometimes thinks fort.XX files are binary.
l1=`grep -a -A 50 "Inequivalent" fort.21 | grep " $a1 " | head -1 | awk '{print $3}' | sed 's/.\{10\}/&\n/g' | head -1`
l2=`grep -a -A 50 "Inequivalent" fort.21 | grep " $a2 " | head -1 | awk '{print $3}' | sed 's/.\{10\}/&\n/g' | head -1`
l3=`grep -a -A 50 "Inequivalent" fort.21 | grep " $a3 " | head -1 | awk '{print $3}' | sed 's/.\{10\}/&\n/g' | head -1`

echo "Input Atom Label | DMACRYS Atom Label"
echo "-------------------------------------"
echo "$a1               | $l1"
echo "$a2               | $l2"
echo "$a3               | $l3"

echo "MOLX 1" > axis
echo "X LINE   $l1 $l2 1" >> axis
echo "Y PLANE  $l1 $l2 1 $l3 2" >> axis
echo "ENDS" >> axis

cd $home

echo "Step 3: Finished!"

##################################
# 
# Step 3: Get Coordinates
# 
# ... by re-running neighcrys (with axes file) to generate .dmain file (which contains coordinates)
# (it is also possible to standardize the lengths of bonds to hydrogen's in this step)
#
##################################

echo "Step 3: Getting coordinates ..."

if [ ! -d neighcrys2 ]; then mkdir neighcrys2; fi
cp neighcrys1/axis $mol.$ext cutoff neighcrys2
cd neighcrys2
sed "s/DSPMOL/$mol/g" $dmacrysdir/fort_template2W99.22 > fort.22
sed -i "s/DSPEXT/$ext/g" fort.22

#Run neighcrys using non-interactive mode (commands are read from fort.22)
$neighcrys << EOF
n
EOF

#Un-useless neighcrys (which prints either ${mol}.res.dmain or ${mol}.dmain, depending on whether the input file is in .res or .fdat format !?)
for i in `ls *.$ext.*`; do j=`echo $i | sed 's/\.'$ext'//g'`; mv $i $j; done

cd $home

echo "Step 3: Finished!"

##################################
# 
# Step 4: Set up Gaussian calculation
#
##################################

echo "Step 4: Setting up Gaussian calculation ..."

if [ ! -d gaussian ]; then mkdir gaussian; fi
cd gaussian

# Formatting file to match to the requirements of dmacrys_w99_coord2gaussian.py. Remove or comment out this block to work with the FIT potential in DMACRYS.
cp ../neighcrys2/fort.21 ./
cat -s fort.21 > tmp
mv fort.21 fort.21_original
cat tmp > fort.21

#Extract coordinates to .geom and .xyz files
python $dmacrysdir/dmacrys_w99_coord2gaussian.py fort.21 $mol

#Backup xyz file
mkdir XYZ
cp *.xyz XYZ/

#Create gaussian .com input file from .xyz file

for fname in `ls *.xyz`

do

	echo  %mem=4gb >> $fname.com # Memory available for Gaussian
	echo  %NProcshared=12 >> $fname.com # Processors available to Gaussian
	echo "#" $g03method SP SCF='(tight)'Density=MP2 nosymm Formcheck=all >> $fname.com   #Edit these variables if using something other than MP2 Density=MP2 Formcheck=all
	echo >> $fname.com
	echo $fname >>$fname.com
	echo >> $fname.com
	echo 0 1 >> $fname.com # Gaussians charge and multiplicity
	sed -n 3,500p $fname >> $fname.com # If the xyz file exceeds 500 lines increase the 500p setting.
	echo >> $fname.com

done

for i in `ls *xyz.com | sed 's/\.xyz\.com// g'`;
do
	echo $i
	mv $i.xyz.com $i.com

done


#Run Gaussian

echo waiting for Gaussian PID to end.
date
g03run ${mol}.com # Edit to run the correct version of Gaussian

cd $home
echo "Step 4: Finished!"

##################################
#
# Step 5: Do GDMA
#
##################################

echo "Step 5: Doing GDMA ..."
if [ ! -d gdma ]; then mkdir gdma; fi
cd gdma
sed "s/DSPMOL/$mol/g" $dmacrysdir/data_template > data
cp ../gaussian/Test.FChk ./
echo waiting for GDMA PID to end.
gdma < data

cp $mol.old $mol.punch
python ~/DMACRYS_Scripts/dmacrys_punchlabels.py $mol.old ../gaussian/$mol.geom # Change path to the path where the python script is on your machine.
cp  new.punch $mol.punch
cd $home
echo "Step 5: Finished!"

##################################
#
# Step 6: Do neighcrys
#
##################################

echo "Step 6: Doing neighcrys ..."
if [ ! -d neighcrys3 ]; then mkdir neighcrys3; fi
cp cutoff $mol.$ext ./neighcrys2/axis ./gdma/new.punch neighcrys3
cp ~/Scripts/DMACRYS_Scripts/will01Cl.pots neighcrys3/
cd neighcrys3
sed "s/DSPMOL/$mol/g" $dmacrysdir/fort_template3W99.22 > fort.22
sed -i "s/DSPEXT/$ext/g" fort.22

#Run neighcrys using non-interactive mode (commands are read from fort.22)
$neighcrys << EOF
n
EOF

#Un-useless neighcrys (which prints either ${mol}.res.dmain or ${mol}.dmain, depending on whether the input file is in .res or .fdat format)
for i in `ls *.$ext.*`; do j=`echo $i | sed 's/\.'$ext'//g'`; mv $i $j; done

cd $home
echo "Step 6: Finished!"


##################################
#
# Step 7: Calculate lattice energy
#
##################################

echo "Step 7: Calculating lattice energy ..."
if [ ! -d dmacrys ]; then mkdir dmacrys; fi
cd dmacrys
cp ../neighcrys3/$mol.dmain ../neighcrys3/fort.20 .

echo waiting for DMACRYS plut lattice mimisation PID.
#Run dmacrys
$dmacrys <$mol.dmain> $mol.dmaout

cd $home
echo "Step 7: Finished!"


##################################
#
# Step 8: Set up properties calculation
#
##################################
echo "Step 8: Setting up properties calculation ..."
if [ ! -d neighcrys_props ]; then mkdir neighcrys_props; fi

cd neighcrys_props
cp ../dmacrys/*.dmain ./ 
cp ../dmacrys/fort.20 ./

#Change options
$dmacrysdir/dmacrys_prop.sh
cp $mol.dmain ${mol}_min.dmain

cd $home
echo "Step 8: Finished!"


##################################
#
# Step 9: Calculate properties
#
##################################

echo "Step 9: Calculating properties ..."
if [ ! -d dmacrys_props ]; then mkdir dmacrys_props; fi
cd dmacrys_props
cp ../neighcrys_props/${mol}_min.dmain ../neighcrys_props/fort.20 .
sed -i 's/STAR PLUT/STAR PROP/g' ${mol}_min.dmain

#Run dmacrys
echo waiting for DMACRYS prop lattice mimisation PID.
$dmacrys <${mol}_min.dmain> ${mol}_min.dmaout

#calculate entropies from phonon modes
perl $fvibsetup < ${mol}_min.dmaout > fort.10
$elastphon
$dmacrysdir/done_dmacrys.sh
cp dmacrys.end ../


cd $home
echo "Step 9: Finished!"

##################################
#
# End
#
#################################

#Write log file
done 2>&1 | tee dmacrys_setup_w99.log


